using System;
using System.Collections;
using System.IO;
using System.Runtime.Serialization;
using System.Runtime.Serialization.Formatters.Binary;

namespace TableTree
{
	/// <summary>
	/// Zusammenfassung fr TableTree.
	/// </summary>
	/// 

	public abstract class TableTree
	{
		public uint runCounter = 0;
		public CellNode [] [,,] nodeBaum;
		public byte[,,] lstate;
		public CellNode [] [,,] nodeBaumBack;
		public CellLeaf [,,] leafsBack;
		byte[,,] lstateBack;
		public int baumtiefe = 0;
		int borderLenght;
		
		const byte BMaskChildAll=255;
		const byte BMaskChildLowLeftFront=1;
		const byte BMaskChildLowLeftBack=2;
		const byte BMaskChildLowRightFront=4;
		const byte BMaskChildLowRightBack=8;
		const byte BMaskChildUpLeftFront=16;
		const byte BMaskChildUpLeftBack=32;
		const byte BMaskChildUpRightFront=64;
		const byte BMaskChildUpRightBack=128;
		
		const byte BMaskChildRightHalf=204;
		const byte BMaskChildLeftHalf=51;
		const byte BMaskChildUpHalf=240;
		const byte BMaskChildLowHalf=15;
		const byte BMaskChildFrontHalf=85;
		const byte BMaskChildBackHalf=170;
		
		const byte BMaskChildLowLeftEdge=3;
		const byte BMaskChildLowRightEdge=12;
		const byte BMaskChildUpRightEdge=192;
		const byte BMaskChildUpLeftEdge=48;
		const byte BMaskChildLowFrontEdge=5;
		const byte BMaskChildLowBackEdge=10;
		const byte BMaskChildUpFrontEdge=80;
		const byte BMaskChildUpBackEdge=160;
		const byte BMaskChildFrontLeftEdge=17;
		const byte BMaskChildFrontRightEdge=68;
		const byte BMaskChildBackLeftEdge=34;
		const byte BMaskChildBackRightEdge=136;
		
		private byte [,,] neigFieldBack = new byte[3,3,3];
		private delegate void NeibConnectios (int xo, int yo, int zo, int borderLenght, int l, bool [,,] childs);
		private NeibConnectios conections;
		public enum Direction : byte
		{
			Left,
			Right,
			Top,
			Botton,
			Back,
			Front,
			BordLeftTop,
			BordLeftFront,
			BordLeftBotton,
			BordLeftBack,
			BordTopBack,
			BordTopFront,
			BordBottonFront,
			BordBottonBack,
			BordRightTop,
			BordRightFront,
			BordRightBotton,
			BordRightBack,
			CorLeftTopBack,
			CornLeftTopFront,
			CorLeftBottonFront,
			CorLeftBottonBack,
			CorRightTopBack,
			CroRightTopFront,
			CorRightBottonFront,
			CorRightBottonBack
		};
		

		public byte RightNeibState
		{
			get{return neigFieldBack[2, 1, 1];}
		}

		public byte LeftNeibState
		{
			get{return neigFieldBack[0, 1, 1];}
		}

		public byte BottonNeibState
		{
			get
			{
				return neigFieldBack[1, 0, 1];
			}
		}

		public byte TopNeibState
		{
			get
			{
				return neigFieldBack[1, 2, 1];
			}
		}

		public byte FrontNeibState
		{
			get
			{
				return neigFieldBack[1, 1, 0];
			}
		}

		public byte BackNeibState
		{
			get
			{
				return neigFieldBack[1, 1, 2];
			}
		}

		public byte BordBottonFrontNeibState
		{
			get
			{
				return neigFieldBack[1, 0, 0];
			}
		}

		public byte BordBottonBackNeibState
		{
			get
			{
				return neigFieldBack[1, 0, 2];
			}
		}

		public byte BordTopFrontNeibState
		{
			get
			{
				return neigFieldBack[1, 2, 0];
			}
		}

		public byte BordTopBackNeibState
		{
			get
			{
				return neigFieldBack[1, 2, 2];
			}
		}

		public byte BordLeftFrontNeibState
		{
			get
			{
				return neigFieldBack[0, 1, 0];
			}
		}

		public byte BordLeftBackeNeibState
		{
			get
			{
				return neigFieldBack[0, 1, 2];
			}
		}

		public byte BordRightFrontNeibState
		{
			get
			{
				return neigFieldBack[2, 1, 0];
			}
		}

		public byte BordRightBackNeibState
		{
			get
			{
				return neigFieldBack[2, 1, 2];
			}
		}

		public byte BordLeftBottonNeibState
		{
			get
			{
				return neigFieldBack[0, 0, 1];
			}
		}

		public byte BordLeftTopNeibState
		{
			get
			{
				return neigFieldBack[0, 2, 1];
			}
		}

		public byte BordRightBottonNeibState
		{
			get
			{
				return neigFieldBack[2, 0, 1];
			}
		}

		public byte BordRightTopNeibState
		{
			get
			{
				return neigFieldBack[2, 2, 1];
			}
		}

		public byte CornLeftBottonFrontNeibState
		{
			get
			{
				return neigFieldBack[0, 0, 0];
			}
		}

		public byte CornLeftBottonBackNeibState
		{
			get
			{
				return neigFieldBack[0, 0, 2];
			}
		}

		public byte CornLeftTopFrontNeibState
		{
			get
			{
				return neigFieldBack[0, 2, 0];
			}
		}

		public byte CornLeftTopBackNeibState
		{
			get
			{
				return neigFieldBack[0, 2, 2];
			}
		}

		public byte CornRightBottonBackNeibState
		{
			get
			{
				return neigFieldBack[2, 0, 2];
			}
		}

		public byte CornRightBottonFrontNeibState
		{
			get
			{
				return neigFieldBack[2, 0, 0];
			}
		}

		public byte CornRightTopBackNeibState
		{
			get
			{
				return neigFieldBack[2, 2, 2];
			}
		}

		public byte CornRightTopFrontNeibState
		{
			get
			{
				return neigFieldBack[2, 2, 0];
			}
		}


		public TableTree (int baumtiefe)
		{
			this.baumtiefe=baumtiefe;
			nodeBaum=new CellNode[baumtiefe][,,];
			SetNeibConnections(GiveUsedNeibDirection());
			for(int level = 0; level<this.baumtiefe; level++)
			{
				borderLenght=(int)System.Math.Pow(2, level);
				nodeBaum[level]=new CellNode[borderLenght, borderLenght, borderLenght];

				for(int i = 0; i<borderLenght; i++)
				{
					for(int j = 0; j<borderLenght; j++)
					{
						for(int k = 0; k<borderLenght; k++)
						{
							nodeBaum[level][i, j, k].changed=BMaskChildAll;
						}
					}
				}
			}

			borderLenght=(int)System.Math.Pow(2, this.baumtiefe);
			lstate=new byte[borderLenght, borderLenght, borderLenght];

			/*for(int i = 0; i<borderLenght; i++)
			{
				for(int j = 0; j<borderLenght; j++)
				{
					for(int k = 0; k<borderLenght; k++)
					{
						lstate[i, j, k]=0;
					}
				}
			}*/
		}
		private void SetNeibConnections (byte []directions)
		{
			foreach(byte direction in directions)
			{
				if(direction==0)
					conections+=new NeibConnectios(CheckNeiBLeft);

				if(direction==1)
					conections+=new NeibConnectios(CheckNeiBRight);

				if(direction==2)
					conections+=new NeibConnectios(CheckNeiBTop);

				if(direction==3)
					conections+=new NeibConnectios(CheckNeiBBotton);

				if(direction==4)
					conections+=new NeibConnectios(CheckNeiBBack);

				if(direction==5)
					conections+=new NeibConnectios(CheckNeiBFront);

				if(direction==6)
					conections+=new NeibConnectios(CheckNeiBBordLeftTop);

				if(direction==7)
					conections+=new NeibConnectios(CheckNeiBBordLeftFront);

				if(direction==8)
					conections+=new NeibConnectios(CheckNeiBBordLeftBotton);

				if(direction==9)
					conections+=new NeibConnectios(CheckNeiBBordLeftBack);

				if(direction==10)
					conections+=new NeibConnectios(CheckNeiBBordTopBack);

				if(direction==11)
					conections+=new NeibConnectios(CheckNeiBBordTopFront);

				if(direction==12)
					conections+=new NeibConnectios(CheckNeiBBordBottonFront);

				if(direction==13)
					conections+=new NeibConnectios(CheckNeiBBordBottonBack);

				if(direction==14)
					conections+=new NeibConnectios(CheckNeiBBordRightTop);

				if(direction==15)
					conections+=new NeibConnectios(CheckNeiBBordRightFront);

				if(direction==16)
					conections+=new NeibConnectios(CheckNeiBBordRightBotton);

				if(direction==17)
					conections+=new NeibConnectios(CheckNeiBBordRightBack);

				if(direction==18)
					conections+=new NeibConnectios(CheckNeiBCorLeftTopBack);

				if(direction==19)
					conections+=new NeibConnectios(CheckNeiBCornLeftTopFront);

				if(direction==20)
					conections+=new NeibConnectios(CheckNeiBCorLeftBottonFront);

				if(direction==21)
					conections+=new NeibConnectios(CheckNeiBCorLeftBottonBack);

				if(direction==22)
					conections+=new NeibConnectios(CheckNeiBCorRightTopBack);

				if(direction==23)
					conections+=new NeibConnectios(CheckNeiBCroRightTopFront);

				if(direction==24)
					conections+=new NeibConnectios(CheckNeiBCorRightBottonFront);

				if(direction==25)
					conections+=new NeibConnectios(CheckNeiBCorRightBottonBack);
			}
		}


		public byte CycleStart ()
		{
			byte changed = 0;

			lstateBack=(byte [,,] )lstate.Clone();
			nodeBaumBack=new CellNode[baumtiefe][,,];
			//Serialisation();
			for(int level = 0; level<this.baumtiefe; level++)
			{
				nodeBaumBack[level]=(CellNode [,,] )nodeBaum[level].Clone();
           	}
			Win32.HiPerfTimer timmer = new Win32.HiPerfTimer();
			timmer.Start();
			MoveDown(0, 0, 0, 0, ref changed);
			//Serialisation();
			timmer.Stop();
			Console.WriteLine("zeit: "+timmer.Duration);
			
			/*borderLenght = (int)System.Math.Pow(2,this.baumtiefe);
			for (int level=0; level<this.baumtiefe; level++)
			{
				for (int i=0; i< (int)System.Math.Pow(2,level); i++)
				{
					for (int j=0; j< (int)System.Math.Pow(2,level); j++)
					{
						for (int k=0; k< (int)System.Math.Pow(2,level); k++)
						{
							if(nodeBaum[level][i,j,k].changed!=0)
							{
								Console.WriteLine("level " + level+","+ i+","+j+","+k+": "+nodeBaumBack[level][i,j,k].changed);
								Console.WriteLine("level " + level+","+ i+","+j+","+k+": "+nodeBaum[level][i,j,k].changed);
								//Console.WriteLine();
							}
						}
					}
				}
				Console.WriteLine();
				Console.ReadLine();
			}
			for (int i=0; i< borderLenght; i++)
			{
				for (int j=0; j< borderLenght; j++)
				{
					for (int k=0; k< borderLenght; k++)
					{
						
						//Console.WriteLine("leafs "+i+","+j+","+k+": "+leafs[i,j,k].changed);
						//Console.WriteLine("leafsBack "+ i+","+j+","+k+": "+leafsBack[i,j,k].changed);
						Console.WriteLine("leafs "+i+","+j+","+k+": "+lstate[i,j,k]);
						//Console.WriteLine("leafsBack "+ i+","+j+","+k+": "+leafsBack[i,j,k].updRate);
                     
					}
				}Console.ReadLine();
			}*/

			runCounter++;
			return changed;
		}

		public void MoveDown (int l, int x, int y, int z, ref byte changed)
		{
			//Console.WriteLine("LEVEL: "+l);
			int borderLenght = (int)System.Math.Pow(2, l);
			int borderLLeaf = (int)System.Math.Pow(2, baumtiefe);
			int xo = x+borderLenght;
			int yo = y+borderLenght;
			int zo = z+borderLenght;
			bool [,,] childs = new bool[2, 2, 2];

			byte changedB = 0;
			bool changedBB=false;

			
            if((nodeBaumBack[l][x,y,z].changed)==0)
			{
				conections(xo, yo, zo, borderLenght, l, childs);
			}
			else
			{
				childs[0, 0, 0]=true;//(nodeBaumBack[l][x, y, z].changed&BMaskChildLowLeftFront)!=0;
				childs[0, 0, 1]=true;//(nodeBaumBack[l][x, y, z].changed&BMaskChildLowLeftBack)!=0;
				childs[0, 1, 0]=true;//(nodeBaumBack[l][x, y, z].changed&BMaskChildUpLeftFront)!=0;
				childs[0, 1, 1]=true;//(nodeBaumBack[l][x, y, z].changed&BMaskChildUpLeftBack)!=0;
				childs[1, 0, 0]=true;//(nodeBaumBack[l][x, y, z].changed&BMaskChildLowRightFront)!=0;
				childs[1, 0, 1]=true;//(nodeBaumBack[l][x, y, z].changed&BMaskChildLowRightBack)!=0;
				childs[1, 1, 0]=true;//(nodeBaumBack[l][x, y, z].changed&BMaskChildUpRightFront)!=0;
				childs[1, 1, 1]=true;//(nodeBaumBack[l][x, y, z].changed&BMaskChildUpRightBack)!=0;
			}

			if(l+1<baumtiefe)
			{
				if(childs[0, 0, 0])
				{
					changedB=0;
					MoveDown(l+1, 2*x, 2*y, 2*z, ref changedB);
					if(changedB!=0)
					changed=(byte)(changed | BMaskChildLowLeftFront);
				}

				if(childs[0, 0, 1])
				{
					changedB=0;
					MoveDown(l+1, 2*x, 2*y, 2*z+1, ref changedB);
					if(changedB!=0)
					changed=(byte)(changed | BMaskChildLowLeftBack);
				}

				if(childs[0, 1, 0])
				{
					changedB=0;
					MoveDown(l+1, 2*x, 2*y+1, 2*z, ref changedB);
					if(changedB!=0)
					changed=(byte)(changed | BMaskChildUpLeftFront);
				}

				if(childs[0, 1, 1])
				{
					changedB=0;
					MoveDown(l+1, 2*x, 2*y+1, 2*z+1, ref changedB);
					if(changedB!=0)
					changed=(byte)(changed | BMaskChildUpLeftBack);
				}

				if(childs[1, 0, 0])
				{
					changedB=0;
					MoveDown(l+1, 2*x+1, 2*y, 2*z, ref changedB);
					if(changedB!=0)
					changed=(byte)(changed | BMaskChildLowRightFront);
				}

				if(childs[1, 0, 1])
				{
					changedB=0;
					MoveDown(l+1, 2*x+1, 2*y, 2*z+1, ref changedB);
					if(changedB!=0)
					changed=(byte)(changed | BMaskChildLowRightBack);
				}

				if(childs[1, 1, 0])
				{
					changedB=0;
					MoveDown(l+1, 2*x+1, 2*y+1, 2*z, ref changedB);
					if(changedB!=0)
					changed=(byte)(changed | BMaskChildUpRightFront);
				}

				if(childs[1, 1, 1])
				{
					changedB=0;
					MoveDown(l+1, 2*x+1, 2*y+1, 2*z+1, ref changedB);
					if(changedB!=0)
					changed=(byte)(changed | BMaskChildUpRightBack);
				}

				nodeBaum[l][x, y, z].changed=changed;
			}
			else
			{

				if(childs[0, 0, 0])
				{
					neighbourfield(2*x, 2*y, 2*z);
					changedBB=false;
					UpdateState(ref changedBB,2*x, 2*y, 2*z);
					if(changedBB)
					changed=(byte)(changed | BMaskChildLowLeftFront);
				}

				if(childs[0, 0, 1])
				{
					neighbourfield(2*x, 2*y, 2*z+1);
					changedBB=false;
					UpdateState(ref changedBB,2*x, 2*y, 2*z+1);
					if(changedBB)
					changed=(byte)(changed | BMaskChildLowLeftBack);
				}

				if(childs[0, 1, 0])
				{
					neighbourfield(2*x, 2*y+1, 2*z);
					changedBB=false;
					UpdateState(ref changedBB,2*x, 2*y+1, 2*z);
					if(changedBB)
					changed=(byte)(changed | BMaskChildUpLeftFront);
				}

				if(childs[0, 1, 1])
				{
					neighbourfield(2*x, 2*y+1, 2*z+1);
					changedBB=false;
					UpdateState(ref changedBB,2*x, 2*y+1, 2*z+1);
					if(changedBB)
					changed=(byte)(changed | BMaskChildUpLeftBack);
				}

				if(childs[1, 0, 0])
				{
					neighbourfield(2*x+1, 2*y, 2*z);
					changedBB=false;
					UpdateState(ref changedBB,2*x+1, 2*y, 2*z);
					if(changedBB)
					changed=(byte)(changed | BMaskChildLowRightFront);
				}

				if(childs[1, 0, 1])
				{
					neighbourfield(2*x+1, 2*y, 2*z+1);
					changedBB=false;
					UpdateState(ref changedBB,2*x+1, 2*y, 2*z+1);
					if(changedBB)
					changed=(byte)(changed | BMaskChildLowRightBack);
				}

				if(childs[1, 1, 0])
				{
					neighbourfield(2*x+1, 2*y+1, 2*z);
					changedBB=false;
					UpdateState(ref changedBB,2*x+1, 2*y+1, 2*z);
					if(changedBB)
					changed=(byte)(changed | BMaskChildUpRightFront);
				}

				if(childs[1, 1, 1])
				{
					neighbourfield(2*x+1, 2*y+1, 2*z+1);
					changedBB=false;
					UpdateState(ref changedBB,2*x+1, 2*y+1, 2*z+1);
					if(changedBB)
					changed=(byte)(changed | BMaskChildUpRightBack);
				}

				nodeBaum[l][x, y, z].changed=changed;
			}
		}
		private void CheckNeiBRight (int xo, int yo, int zo, int borderLenght, int l, bool [,,] childs)
		{
			if(((nodeBaumBack[l][ (xo+1)%borderLenght, (yo)%borderLenght,(zo)%borderLenght].changed)
			     &BMaskChildLeftHalf)!=0)
			{
				childs[1, 0, 0]=true;
				childs[1, 0, 1]=true;
				childs[1, 1, 0]=true;
				childs[1, 1, 1]=true;
			}
		}

		private void CheckNeiBLeft (int xo, int yo, int zo, int borderLenght, int l, bool [,,] childs)
		{
			if(((nodeBaumBack[l][ (xo-1)%borderLenght, (yo)%borderLenght,(zo)%borderLenght].changed)
			&BMaskChildRightHalf)!=0)
			{
				childs[0, 0, 0]=true;
				childs[0, 0, 1]=true;
				childs[0, 1, 0]=true;
				childs[0, 1, 1]=true;
			}
		}

		private void CheckNeiBTop (int xo, int yo, int zo, int borderLenght, int l, bool [,,] childs)
		{
			if(((nodeBaumBack[l][ (xo)%borderLenght, (yo+1)%borderLenght,(zo)%borderLenght].changed)
			&BMaskChildLowHalf)!=0)
			{
				childs[0, 1, 0]=true;
				childs[0, 1, 1]=true;
				childs[1, 1, 0]=true;
				childs[1, 1, 1]=true;
			}
		}

		private void CheckNeiBBotton (int xo, int yo, int zo, int borderLenght, int l, bool [,,] childs)
		{
			if(((nodeBaumBack[l][(xo)%borderLenght, (yo-1)%borderLenght,(zo)%borderLenght].changed)
			&BMaskChildUpHalf)!=0)
			{
				childs[0, 0, 0]=true;
				childs[0, 0, 1]=true;
				childs[1, 0, 0]=true;
				childs[1, 0, 1]=true;
			}
		}

		private void CheckNeiBBack (int xo, int yo, int zo, int borderLenght, int l, bool [,,] childs)
		{
			if(((nodeBaumBack[l][ (xo)%borderLenght, (yo)%borderLenght,(zo+1)%borderLenght].changed)
			&BMaskChildFrontHalf)!=0)
			{
				childs[0, 0, 1]=true;
				childs[0, 1, 1]=true;
				childs[1, 0, 1]=true;
				childs[1, 1, 1]=true;
			}
		}

		private void CheckNeiBFront (int xo, int yo, int zo, int borderLenght, int l, bool [,,] childs)
		{
			if(((nodeBaumBack[l][ (xo)%borderLenght, (yo)%borderLenght,(zo-1)%borderLenght].changed)
			&BMaskChildBackHalf)!=0)
			{
				childs[0, 0, 0]=true;
				childs[0, 1, 0]=true;
				childs[1, 0, 0]=true;
				childs[1, 1, 0]=true;
			}
		}

		private void CheckNeiBBordLeftFront (int xo, int yo, int zo, int borderLenght, int l, bool [,,] childs)
		{
			if(((nodeBaumBack[l][ (xo-1)%borderLenght, (yo)%borderLenght,(zo-1)%borderLenght].changed)
			&BMaskChildBackRightEdge)!=0)
			{
				childs[0, 1, 0]=true;
				childs[0, 0, 0]=true;
			}
		}

		private void CheckNeiBBordLeftBotton (int xo, int yo, int zo, int borderLenght, int l, bool [,,] childs)
		{
			if(((nodeBaumBack[l][ (xo-1)%borderLenght, (yo-1)%borderLenght,(zo)%borderLenght].changed)
			&BMaskChildUpRightEdge)!=0)
			{
				childs[0, 0, 1]=true;
				childs[0, 0, 0]=true;
			}
		}

		private void CheckNeiBBordLeftBack (int xo, int yo, int zo ,int borderLenght, int l, bool [,,] childs)
		{
			if(((nodeBaumBack[l][ (xo-1)%borderLenght, (yo)%borderLenght,(zo+1)%borderLenght].changed)
			&BMaskChildFrontRightEdge)!=0)
			{
				childs[0, 1, 1]=true;
				childs[0, 0, 1]=true;
			}
		}

		private void CheckNeiBBordLeftTop (int xo, int yo, int zo ,int borderLenght, int l, bool [,,] childs)
		{
			if(((nodeBaumBack[l][ (xo-1)%borderLenght, (yo+1)%borderLenght,(zo)%borderLenght].changed)
			&BMaskChildLowRightEdge)!=0)
			{
				childs[0, 1, 0]=true;
				childs[0, 1, 1]=true;
			}
		}

		private void CheckNeiBBordRightBotton (int xo, int yo, int zo, int borderLenght, int l, bool [,,] childs)
		{
			if(((nodeBaumBack[l][ (xo+1)%borderLenght, (yo-1)%borderLenght,(zo)%borderLenght].changed)
			&BMaskChildUpLeftEdge)!=0)
			{
				childs[1, 0, 0]=true;
				childs[1, 0, 1]=true;
			}
		}

		private void CheckNeiBBordRightBack (int xo, int yo, int zo, int borderLenght, int l, bool [,,] childs)
		{
			if(((nodeBaumBack[l][ (xo+1)%borderLenght, (yo)%borderLenght,(zo+1)%borderLenght].changed)
			&BMaskChildFrontLeftEdge)!=0)
			{
				childs[1, 0, 1]=true;
				childs[1, 1, 1]=true;
			}
		}

		private void CheckNeiBBordRightFront (int xo, int yo, int zo, int borderLenght, int l, bool [,,] childs)
		{
			if(((nodeBaumBack[l][ (xo+1)%borderLenght, (yo)%borderLenght,(zo-1)%borderLenght].changed)
			&BMaskChildBackLeftEdge)!=0)
			{
				childs[1, 0, 0]=true;
				childs[1, 1, 0]=true;
			}
		}

		private void CheckNeiBBordRightTop (int xo, int yo, int zo, int borderLenght, int l, bool [,,] childs)
		{
			if(((nodeBaumBack[l][ (xo+1)%borderLenght, (yo+1)%borderLenght,(zo)%borderLenght].changed)
			&BMaskChildLowLeftEdge)!=0)
			{
				childs[1, 1, 0]=true;
				childs[1, 1, 1]=true;
			}
		}

		private void CheckNeiBBordTopBack (int xo, int yo, int zo, int borderLenght, int l, bool [,,] childs)
		{
			if(((nodeBaumBack[l][ (xo)%borderLenght, (yo+1)%borderLenght,(zo+1)%borderLenght].changed)
			&BMaskChildLowFrontEdge)!=0)
			{
				childs[0, 1, 1]=true;
				childs[1, 1, 1]=true;
			}
		}

		private void CheckNeiBBordTopFront (int xo, int yo, int zo, int borderLenght, int l, bool [,,] childs)
		{
			if(((nodeBaumBack[l][ (xo)%borderLenght, (yo+1)%borderLenght,(zo-1)%borderLenght].changed)
			&BMaskChildLowBackEdge)!=0)
			{
				childs[0, 1, 0]=true;
				childs[1, 1, 0]=true;
			}
		}

		private void CheckNeiBBordBottonBack (int xo, int yo, int zo, int borderLenght, int l, bool [,,] childs)
		{
			if(((nodeBaumBack[l][ (xo)%borderLenght, (yo-1)%borderLenght,(zo+1)%borderLenght].changed)
			&BMaskChildUpFrontEdge)!=0)
			{
				childs[0, 0, 1]=true;
				childs[1, 0, 1]=true;
			}
		}

		private void CheckNeiBBordBottonFront (int xo, int yo, int zo, int borderLenght, int l, bool [,,] childs)
		{
			if(((nodeBaumBack[l][ (xo)%borderLenght, (yo-1)%borderLenght,(zo-1)%borderLenght].changed)
			&BMaskChildUpBackEdge)!=0)
			{
				childs[0, 0, 0]=true;
				childs[1, 0, 0]=true;
			}
		}

		private void CheckNeiBCorLeftTopBack (int xo, int yo, int zo, int borderLenght, int l, bool [,,] childs)
		{
			if(((nodeBaumBack[l][ (xo-1)%borderLenght, (yo+1)%borderLenght,(zo+1)%borderLenght].changed)
			&BMaskChildLowRightFront)!=0)
			{
				childs[0, 1, 1]=true;
			}
		}

		private void CheckNeiBCornLeftTopFront (int xo, int yo, int zo, int borderLenght, int l, bool [,,] childs)
		{
			if(((nodeBaumBack[l][ (xo-1)%borderLenght, (yo+1)%borderLenght,(zo-1)%borderLenght].changed)
			&BMaskChildLowRightBack)!=0)
			{
				childs[0, 1, 0]=true;
			}
		}

		private void CheckNeiBCorLeftBottonBack (int xo, int yo, int zo, int borderLenght, int l, bool [,,] childs)
		{
			if(((nodeBaumBack[l][ (xo-1)%borderLenght, (yo-1)%borderLenght,(zo+1)%borderLenght].changed)
			&BMaskChildUpRightFront)!=0)
			{
				childs[0, 0, 1]=true;
			}
		}

		private void CheckNeiBCorLeftBottonFront (int xo, int yo, int zo, int borderLenght, int l, bool [,,] childs)
		{
			if(((nodeBaumBack[l][ (xo-1)%borderLenght, (yo-1)%borderLenght,(zo-1)%borderLenght].changed)
			&BMaskChildUpRightBack)!=0)
			{
				childs[0, 0, 0]=true;
			}
		}

		private void CheckNeiBCorRightTopBack (int xo, int yo, int zo, int borderLenght, int l, bool [,,] childs)
		{
			if(((nodeBaumBack[l][ (xo+1)%borderLenght, (yo+1)%borderLenght,(zo+1)%borderLenght].changed)
			&BMaskChildLowLeftFront)!=0)
			{
				childs[1, 1, 1]=true;
			}
		}

		private void CheckNeiBCroRightTopFront (int xo, int yo, int zo, int borderLenght, int l, bool [,,] childs)
		{
			if(((nodeBaumBack[l][ (xo+1)%borderLenght, (yo+1)%borderLenght,(zo-1)%borderLenght].changed)
			&BMaskChildLowLeftBack)!=0)
			{
				childs[1, 1, 0]=true;
			}
		}

		private void CheckNeiBCorRightBottonBack (int xo, int yo, int zo, int borderLenght, int l, bool [,,] childs)
		{
			if(((nodeBaumBack[l][ (xo+1)%borderLenght, (yo-1)%borderLenght,(zo+1)%borderLenght].changed)
			&BMaskChildUpLeftFront)!=0)
			{
				childs[1, 0, 1]=true;
			}
		}

		private void CheckNeiBCorRightBottonFront (int xo, int yo, int zo, int borderLenght, int l, bool [,,] childs)
		{
			if(((nodeBaumBack[l][ (xo+1)%borderLenght, (yo-1)%borderLenght,(zo-1)%borderLenght].changed)
			&BMaskChildUpLeftBack)!=0)
			{
				childs[1, 0, 0]=true;
			}
		}


		private void neighbourfield (int x, int y, int z)
		{
			int borderLLeaf = (int)System.Math.Pow(2, baumtiefe);
			int xo = x+borderLLeaf;
			int yo = y+borderLLeaf;
			int zo = z+borderLLeaf;

			for(int i = -1; i<2; i++)
			{
				for(int j = -1; j<2; j++)
				{
					for(int k = -1; k<2; k++)
					{
						this.neigFieldBack[i+1,j+1,k+1]=lstateBack[(xo+i)%borderLLeaf, (yo+j)%borderLLeaf,(zo+k)%borderLLeaf]; 

					}
				}
			}
			

		}

		private void UpdateState (ref bool changed,int x,int y, int z)
		{
			
			SetState(Rules(neigFieldBack[1, 1, 1]), ref changed,x,y,z);
			
		}

		public int CountNeibWhiteState( byte state)
		{ 
			int count = 0;
			for(int i = 0; i<3; i++)
				for(int j = 0; j<3; j++)
					for(int k = 0; k<3; k++)
					{
						if(neigFieldBack[i, j, k]== state)
						{
							count++;
						}
					}
			return count;
		}

		private void SetState (byte state,ref bool changed,int x, int y, int z)
		{
			lstate[x,y,z]=(byte)state;

			if(neigFieldBack[1, 1, 1]!=state)
			{
				changed=true;
			}
			else
			{
				changed=false;
			}

		}

		private void Serialisation()
		{ 
			
			FileStream s = new FileStream("Frame" + runCounter + ".ser",FileMode.Create);
			IFormatter f = new BinaryFormatter();
			f.Serialize(s, lstate);
			s.Close();			
		}
		abstract public byte Rules (byte currentstate);
		abstract public byte[] GiveUsedNeibDirection();
	}
}
